@@ -1,3 +1,5 @@
+"use strict";
+
 Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");
 try {
 	Components.utils.import("resource://gre/modules/AddonManager.jsm");
@@ -21,25 +23,23 @@ StylishStartup.prototype = {
 			AddonManagerPrivate.registerProvider(UserStyleManager, [{
 				id: "userstyle",
 				name: bundle.GetStringFromName("manageaddonstitle"),
-				uiPriority: 7000,
+				uiPriority: 4600,
 				viewType: AddonManager.VIEW_TYPE_LIST
 			}]);
 		}
+		wireUpMessaging();
 	}
 }
 
-// this throws and is unnecessary in firefox 4+
-try {
-Components.classes["@mozilla.org/categorymanager;1"].getService(Components.interfaces.nsICategoryManager).addCategoryEntry("profile-after-change", "StylishStartup", StylishStartup.prototype.contractID, true, true);
-} catch (ex) {}
-
 var turnOnOffObserver = {
 	observe: function(subject, topic, data) {
 		var service = Components.classes["@userstyles.org/style;1"].getService(Components.interfaces.stylishStyle);
-		service.findEnabled(true, subject.QueryInterface(Components.interfaces.nsIPrefBranch2).getBoolPref(data) ? service.REGISTER_STYLE_ON_LOAD : service.UNREGISTER_STYLE_ON_LOAD, {});
+		service.findEnabled(true, subject.QueryInterface(Components.interfaces.nsIPrefBranch).getBoolPref(data) ? service.REGISTER_STYLE_ON_LOAD : service.UNREGISTER_STYLE_ON_LOAD, {});
 	}
 }
 
+var prefService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
+
 var UserStyleManager = {
 
 	getAddonsByTypes: function(aTypes, aCallback) {
@@ -121,25 +121,44 @@ var UserStyleManager = {
 	}
 };
 
-function getUserStyleWrapper(style) {
-	return {
-		id: style.id.toString(),
+function getUserStyleWrapper(s) {
+	var w = {
+		style: s,
 		type: "userstyle",
-		name: style.name,
-		homepageURL: style.url,
 		appDisabled: false,
 		pendingOperations: AddonManager.PENDING_NONE,
 		isCompatible: true,
 		isPlatformCompatible: true,
 		iconURL: "chrome://stylish/skin/32.png",
-		size: style.code.length,
 		scope: AddonManager.SCOPE_PROFILE,
 		blocklistState: Components.interfaces.nsIBlocklistService.STATE_NOT_BLOCKED,
 		pendingOperations: AddonManager.PENDING_NONE,
-		providesUpdatesSecurely: style.updateUrl == null || style.updateUrl == "",
 		version: "",
 		operationsRequiringRestart: AddonManager.OP_NEEDS_RESTART_NONE,
-		styleTypes: style.getTypes({}).sort().join(","),
+
+		get id() {
+			return this.style.id.toString();
+		},
+
+		get name() {
+			return this.style.name;
+		},
+
+		get homepageURL() {
+			return this.style.url;
+		},
+
+		get size() {
+			return this.style.code.length;
+		},
+
+		get providesUpdatesSecurely() {
+			return this.style.updateUrl == null || this.style.updateUrl == "";
+		},
+
+		get styleTypes() {
+			return this.style.getTypes({}).sort().join(",");
+		},
 
 		get optionsURL() {
 			return null;
@@ -147,8 +166,8 @@ function getUserStyleWrapper(style) {
 
 		get permissions() {
 			return AddonManager.PERM_CAN_UNINSTALL | 
-				(style.enabled ? AddonManager.PERM_CAN_DISABLE : AddonManager.PERM_CAN_ENABLE) | 
-				(style.updateUrl != null && style.updateUrl != "" && style.updateUrl.length <= 2000 && Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch).getBoolPref("extensions.stylish.updatesEnabled") ? AddonManager.PERM_CAN_UPGRADE : 0); // if the url length is too long, a GET won't work, and it's probably going to be too much server-side to handle
+				(this.style.enabled ? AddonManager.PERM_CAN_DISABLE : AddonManager.PERM_CAN_ENABLE) |
+				(this.style.updateUrl != null && this.style.updateUrl != "" && this.style.updateUrl.length <= 2000 && prefService.getBoolPref("extensions.stylish.updatesEnabled") ? AddonManager.PERM_CAN_UPGRADE : 0); // if the url length is too long, a GET won't work, and it's probably going to be too much server-side to handle
 		},
 
 		get isActive() {
@@ -156,30 +175,25 @@ function getUserStyleWrapper(style) {
 		},
 
 		get userDisabled() {
-			return !style.enabled;
+			return !this.style.enabled;
 		},
 
 		set userDisabled(val) {
-			style.enabled = !val;
-			style.save();
+			if (this.style.enabled == !val) {
+				// no op
+				return;
+			}
+			this.style.enabled = !val;
+			this.style.save();
 			AddonManagerPrivate.callAddonListeners(val ? "onEnabling" : "onDisabling", this, false);
 		},
 
 		get description() {
-			var tagsA = style.getMeta("tag", {})
-			var tags = "";
-			if (tagsA.length > 0) {
-				tags = bundle.formatStringFromName("tagstyledescription", [tagsA.join(", ")], 1);
-			}
-			var applies = this.getAppliesString();
-			if (applies != "" && tags != "") {
-				return applies + " \n\n" + tags;
-			}
-			return applies + tags;
+			return this.getAppliesString();
 		},
 
 		getAppliesString: function() {
-			var types = style.getTypes({});
+			var types = this.style.getTypes({});
 			if (types.length == 1) {
 				if (types[0] == "global") {
 					return bundle.GetStringFromName("globalstyledescription");
@@ -188,19 +202,7 @@ function getUserStyleWrapper(style) {
 					return bundle.GetStringFromName("appstyledescription");
 				}
 			}
-
-			var service = Components.classes["@userstyles.org/style;1"].getService(Components.interfaces.stylishStyle)
-
-			var domains = style.getMeta("domain", {});
-			var urls = style.getMeta("url", {});
-			var urlPrefixes = style.getMeta("url-prefix", {});
-			var regexps = style.getMeta("regexp", {});
-
-			var affects = domains.concat(urls).concat(urlPrefixes.map(function(u) { 
-				return u + "*"
-			})).concat(regexps.map(function(a) {
-				return service.regexToSample(a);
-			}));
+			var affects = this.style.getPrettyAppliesTo({});
 			if (affects.length > 0) {
 				return bundle.formatStringFromName("sitestyledescription", [affects.join(", ")], 1);
 			}
@@ -208,11 +210,11 @@ function getUserStyleWrapper(style) {
 		},
 
 		uninstall: function() {
-			style.delete();
+			this.style.delete();
 		},
 
 		findUpdates: function(listener, flags) {
-			style.checkForUpdates(getUserStyleUpdateCheckObserver(this, listener));
+			this.style.checkForUpdates(getUserStyleUpdateCheckObserver(this, listener));
 		},
 
 		isCompatibleWith: function(appVersion, platformVersion) {
@@ -220,14 +222,24 @@ function getUserStyleWrapper(style) {
 		},
 
 		get applyBackgroundUpdates() {
-			return parseInt(style.applyBackgroundUpdates);
+			return parseInt(this.style.applyBackgroundUpdates);
 		},
 
 		set applyBackgroundUpdates(abu) {
-			style.applyBackgroundUpdates = abu;
-			style.save();
+			this.style.applyBackgroundUpdates = abu;
+			this.style.save();
+		},
+
+		observe: function(subject, topic, data) {
+			// Update our stuff if the style was changed
+			if (this.style.id == subject.id) {
+				this.style = subject;
+			}
 		}
 	};
+	var observerService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
+	observerService.addObserver(w, "stylish-style-change", false);
+	return w;
 }
 
 // An observer for style update checks.
@@ -279,7 +291,7 @@ function getUserStyleUpdateInstallItem(addonItem) {
 			}, this);
 			var service = Components.classes["@userstyles.org/style;1"].getService(Components.interfaces.stylishStyle);
 			var that = this;
-			
+
 			// Results for "apply updates"
 			var updateAttemptObserver = {
 				observe: function(subject, topic, data) {
@@ -295,7 +307,7 @@ function getUserStyleUpdateInstallItem(addonItem) {
 							AddonManagerPrivate.callInstallListeners("onDownloadFailed", that.listeners, that);
 							break;
 						case "update-success":
-							AddonManagerPrivate.callInstallListeners("onInstallEnded", that.listeners, that);
+							AddonManagerPrivate.callInstallListeners("onInstallEnded", that.listeners, that, that.addon);
 							break;
 					}
 
@@ -335,14 +347,14 @@ var addonsObserver = {
 					name: subject.name,
 					type: "userstyle",
 					state: AddonManager.STATE_INSTALLED,
-					addon: getUserStyleWrapper(subject)
+					addon: itemWrapper
 				};
 				AddonManagerPrivate.callInstallListeners("onNewInstall", [], install);
 				AddonManagerPrivate.callInstallListeners("onInstallStarted", [], install);
 				AddonManagerPrivate.callInstallListeners("onInstallEnded", [], install, itemWrapper);
 				break;
 			case "stylish-style-change":
-				AddonManagerPrivate.callInstallListeners("onExternalInstall", null, itemWrapper, itemWrapper, false);
+				AddonManagerPrivate.callInstallListeners("onExternalInstall", [], itemWrapper, itemWrapper, false);
 				break;
 			case "stylish-style-delete":
 				AddonManagerPrivate.callAddonListeners("onUninstalled", itemWrapper);
@@ -355,10 +367,73 @@ observerService.addObserver(addonsObserver, "stylish-style-add", false);
 observerService.addObserver(addonsObserver, "stylish-style-change", false);
 observerService.addObserver(addonsObserver, "stylish-style-delete", false);
 
-Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).QueryInterface(Components.interfaces.nsIPrefBranch2).addObserver("extensions.stylish.styleRegistrationEnabled", turnOnOffObserver, false);
+Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).QueryInterface(Components.interfaces.nsIPrefBranch).addObserver("extensions.stylish.styleRegistrationEnabled", turnOnOffObserver, false);
+
+function wireUpMessaging() {
+	Components.utils.import("chrome://stylish/content/common.js");
+	var service = Components.classes["@userstyles.org/style;1"].getService(Components.interfaces.stylishStyle);
+	var STRINGS = Components.classes["@mozilla.org/intl/stringbundle;1"].getService(Components.interfaces.nsIStringBundleService).createBundle("chrome://stylish/locale/overlay.properties");
+
+	var globalMM = Components.classes["@mozilla.org/globalmessagemanager;1"].getService(Components.interfaces.nsIMessageListenerManager);
+	globalMM.loadFrameScript("chrome://stylish/content/install-frame-script.js", true);
+
+	function reply(incomingMessage, name, data) {
+		incomingMessage.target.messageManager.sendAsyncMessage(name, data);
+	}
+
+	function messageToWindow(message) {
+		return message.target.ownerDocument.defaultView;
+	}
+
+	globalMM.addMessageListener("stylish:get-style-install-status", function(message) {
+		var style = service.findByUrl(message.data.idUrl, 0);
+		if (style) {
+			if (style.originalMd5 == message.data.md5) {
+				reply(message, "stylish:style-already-installed");
+			} else {
+				reply(message, "stylish:style-can-be-updated");
+			}
+		} else {
+			reply(message, "stylish:style-can-be-installed");
+		}
+	});
+
+	globalMM.addMessageListener("stylish:install-style", function(message) {
+		stylishCommon.installFromStyleInfo(message.data, function(result) {
+			if (result == "installed") {
+				reply(message, "stylish:style-installed");
+			}
+		}, messageToWindow(message));
+	});
+
+	globalMM.addMessageListener("stylish:update-style", function(message) {
+		var style = service.findByUrl(message.data.idUrl, service.REGISTER_STYLE_ON_CHANGE + service.CALCULATE_META);
+		var code = message.data.code;
+		var md5 = message.data.md5;
+		var md5Url = message.data.md5Url;
+		var updateUrl = message.data.updateUrl;
+		if (!style || !code) {
+			return;
+		}
+		var prompt = STRINGS.formatStringFromName("updatestyle", [style.name], 1);
+		var prompts = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);
+		if (prompts.confirmEx(messageToWindow(message), STRINGS.formatStringFromName("updatestyletitle", [], 0), prompt, prompts.BUTTON_POS_0 * prompts.BUTTON_TITLE_IS_STRING + prompts.BUTTON_POS_1 * prompts.BUTTON_TITLE_CANCEL, STRINGS.formatStringFromName("updatestyleok", [], 0), null, null, null, {}) == 0) {
+			style.code = code;
+
+			//we're now in sync with the remote style, so let's set things appropriately
+			style.originalCode = code;
+			style.md5Url = md5Url;
+			style.originalMd5 = md5;
+			style.updateUrl = updateUrl;
+
+			style.save();
+			reply(message, "stylish:style-updated");
+		}
+	});
+
+}
 
 if (XPCOMUtils.generateNSGetFactory)
     var NSGetFactory = XPCOMUtils.generateNSGetFactory([StylishStartup]);
 else
     var NSGetModule = XPCOMUtils.generateNSGetModule([StylishStartup]);
-
