@@ -1,3 +1,5 @@
+"use strict";
+
 Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");
 
 function Style() {
@@ -61,6 +63,7 @@ Style.prototype = {
 
 	list: function(mode, count) {
 		var styles = this.findSql("SELECT * FROM styles;", {}, mode);
+		styles.sort(this.nameSort);
 		count.value = styles.length;
 		return styles;
 	},
@@ -77,6 +80,7 @@ Style.prototype = {
 
 	findEnabled: function(enabled, mode, count) {
 		var styles = this.findSql("SELECT * FROM styles WHERE enabled = :enabled;", {enabled: enabled}, mode);
+		styles.sort(this.nameSort);
 		count.value = styles.length;
 		return styles;
 	},
@@ -84,8 +88,10 @@ Style.prototype = {
 	findForUrl: function(url, includeGlobal, mode, count) {
 		var styles = this.list(mode, {});
 		styles = styles.filter(function(style) {
-			return style.appliesToUrl(url) || (includeGlobal && style.getTypes({}).indexOf("global") > -1);
+			var isGlobal = style.getTypes({}).indexOf("global") > -1
+			return includeGlobal ? (isGlobal || style.appliesToUrl(url)) : (!isGlobal && style.appliesToUrl(url));
 		});
+		styles.sort(this.nameSort);
 		count.value = styles.length;
 		return styles;
 	},
@@ -99,8 +105,13 @@ Style.prototype = {
 			this.bind(statement, "value", value);
 			var styles = [];
 			while (statement.executeStep()) {
-				styles.push(this.find(this.extract(statement, "style_id"), mode, connection));
+				// theoretically we shouldn't find a style that doesn't exist, but who knows?
+				var s = this.find(this.extract(statement, "style_id"), mode, connection);
+				if (s) {
+					styles.push(s);
+				}
 			}
+			styles.sort(this.nameSort);
 			count.value = styles.length;
 			return styles;
 		} catch (ex) {
@@ -127,7 +138,7 @@ Style.prototype = {
 		// everything up to the first regex character
 		var re = /[\.\(\)\[\]]/g;
 		var match;
-		while (match = re.exec(r)) {
+		while ((match = re.exec(r)) !== null) {
 			if (r[match.index - 1] != "\\") {
 				break;
 			}
@@ -347,13 +358,13 @@ Style.prototype = {
 		}
 		return this.regexpRules.some(function(rule) {
 			try {
-				var re = new RegExp(rule);
+				var re = new RegExp(this.ensureFullMatchRegexp(rule));
 			} catch (ex) {
 				//bad regexp
 				return false;
 			}
 			return re.test(url);
-		});
+		}, this);
 	},
 
 	//previewing make it so that the code for this style is always applied, even if it's disabled
@@ -452,7 +463,7 @@ Style.prototype = {
 
 		//if we have a url for a hash, use that
 		if (this.md5Url) {
-			function handleMd5(text) {
+			var handleMd5 = function (text) {
 				if (text.length != 32) {
 					Components.utils.reportError("Could not update '" + that.name + "' - '" + that.md5Url + "' did not return a md5 hash.");
 					notifyDone("no-update-available");
@@ -465,7 +476,7 @@ Style.prototype = {
 			this.download(this.md5Url, handleMd5, handleFailure);
 		//otherwise use the update URL which makes us download the full code
 		} else if (this.updateUrl) {
-			function handleUpdateUrl(text, contentType) {
+			var handleUpdateUrl = function (text, contentType) {
 				if (contentType != "text/css") {
 					Components.utils.reportError("Could not update '" + that.name + "' - '" + that.updateUrl + "' returned content type '" + contentType + "'.");
 					notifyDone("no-update-available");
@@ -666,7 +677,7 @@ Style.prototype = {
 		else if (
 			(hasGlobal && (namespaces.length == 0 || namespaces.indexOf(this.HTMLNS) != -1))
 			|| urlLikeRules.some(function(url) { return genericPattern.test(url) })
-			|| this.regexpRules.some(function(r) { return (new RegExp(r)).test("http://www.somesiteimadeup.com/");})
+			|| this.regexpRules.some(function(r) { return (new RegExp(that.ensureFullMatchRegexp(r))).test("http://www.somesiteimadeup.com/");})
 		)
 			this.addMeta("type", "global");
 		// everything else is site
@@ -693,7 +704,9 @@ Style.prototype = {
 		var registrationMethod = this.calculateRegistrationMethod();
 		// Save what we applied so we know what to remove later
 		this.appliedInfo = [dataUrl, registrationMethod];
-		this.sss.loadAndRegisterSheet(dataUrl, registrationMethod);
+		if (!this.sss.sheetRegistered(dataUrl, registrationMethod)) {
+			this.sss.loadAndRegisterSheet(dataUrl, registrationMethod);
+		}
 	},
 
 	unregister: function() {
@@ -710,11 +723,12 @@ Style.prototype = {
 			unregisterMethod = this.appliedInfo[1];
 		}
 		
-		if (this.sss.sheetRegistered(unregisterUrl, unregisterMethod))
+		if (this.sss.sheetRegistered(unregisterUrl, unregisterMethod)) {
 			this.sss.unregisterSheet(unregisterUrl, unregisterMethod);
 		// ignore unregistered styles if stylish isn't on
-		else if (this.stylishOn)
+		} else if (this.stylishOn) {
 			Components.utils.reportError("Stylesheet is supposed to be unregistered, but it's not registered in the first place.");
+		}
 		this.appliedInfo = null;
 	},
 
@@ -788,12 +802,12 @@ Style.prototype = {
 			closeConnection = true;
 		}
 		var statement = connection.createStatement(sql);
-		for (i in parameters) {
+		for (var i in parameters) {
 			this.bind(statement, i, parameters[i]);
 		}
 		try {
 			var that = this;
-			function e(name) {
+			var e = function (name) {
 				return that.extract(statement, name);
 			};
 			var styles = [];
@@ -951,22 +965,17 @@ Style.prototype = {
 		}, false);
 		// QI it to nsIXMLHttpRequest to open and send the request:
 		request.QueryInterface(Components.interfaces.nsIXMLHttpRequest);
-		request.open("GET", url, true);
-		this.fixXHR(request);
-		request.send(null);
-	},
-
-	fixXHR: function(request) {
-		//only a problem on 1.9 toolkit
-		var appInfo = Components.classes["@mozilla.org/xre/app-info;1"].getService(Components.interfaces.nsIXULAppInfo);
-		var versionChecker = Components.classes["@mozilla.org/xpcom/version-comparator;1"].getService(Components.interfaces.nsIVersionComparator);
-		if (versionChecker.compare(appInfo.version, "1.9") >= 0 && Components.classes["@mozilla.org/webshell;1"]) {
-			//https://bugzilla.mozilla.org/show_bug.cgi?id=437174
-			var ds = Components.classes["@mozilla.org/webshell;1"].createInstance(Components.interfaces.nsIDocShellTreeItem).QueryInterface(Components.interfaces.nsIInterfaceRequestor);
-			ds.itemType = Components.interfaces.nsIDocShellTreeItem.typeContent;
-			request.channel.loadGroup = ds.getInterface(Components.interfaces.nsILoadGroup);
-			request.channel.loadFlags |= Components.interfaces.nsIChannel.LOAD_DOCUMENT_URI;
+		try {
+			// this can fail if URL is malformed
+			request.open("GET", url, true);
+		} catch(ex) {
+			Components.utils.reportError("Could not download URL '" + url + "'.");
+			if (failureCallback) {
+				failureCallback();
+			}
+			return;
 		}
+		request.send(null);
 	},
 
 	get stylishOn() {
@@ -1008,6 +1017,24 @@ Style.prototype = {
 				}
 			}, null
 		);
+	},
+
+	// Make sure a regexp is set to match the entire string, as opposed to just a portion
+	ensureFullMatchRegexp: function(pattern) {
+		if (pattern == null) {
+			return pattern;
+		}
+		if (pattern[0] != "^") {
+			pattern = "^" + pattern;
+		}
+		if (pattern[pattern.length - 1] != "$") {
+			pattern = pattern + "$";
+		}
+		return pattern;
+	},
+
+	nameSort: function(a, b) {
+		return a.name.localeCompare(b.name);
 	}
 
 };
