@@ -1,20 +1,18 @@
+"use strict";
+
 Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");
 Components.utils.import("resource://gre/modules/Services.jsm");
 var require = null;
-var autocompleter = null;
 try {
 	require = Components.utils.import("resource://gre/modules/devtools/Loader.jsm", {}).devtools.require;
-	autocompleter = require("devtools/sourceeditor/autocomplete");
 } catch (ex) {
 	// file not available...
 }
 
-var saved = false;
+var skipConfirmClosePrompt = false;
 var style = null;
 var strings = null;
-var codeE, nameE, tagsE, updateUrlE;
-var installPingURL = null;
-var installCallback = null;
+var codeE, nameE, updateUrlE;
 //because some editors can have different CRLF settings than what we've saved as, we'll only save if the code in the editor has changed. this will prevent update notifications when there are none
 var initialCode;
 var prefs = Services.prefs.getBranch("extensions.stylish.");
@@ -22,141 +20,169 @@ var prefs = Services.prefs.getBranch("extensions.stylish.");
 const CSSXULNS = "@namespace url(http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul);";
 const CSSHTMLNS = "@namespace url(http://www.w3.org/1999/xhtml);";
 
+// Because the edit windows have different URL, we need to do this ourselves to persist the position for all edit windows.
+// Only do this if we're opened with openDialog, in which case window.opener is set
+if (window.opener) {
+	var windowPersist = JSON.parse(prefs.getCharPref("editorWindowPersist"));
+	window.moveTo(windowPersist.screenX, windowPersist.screenY);
+	window.resizeTo(windowPersist.width, windowPersist.height);
+	if (windowPersist.windowState == 1) {
+		//https://bugzilla.mozilla.org/show_bug.cgi?id=1079962
+		window.addEventListener("load", function() {
+			setTimeout(function() {
+				window.maximize();
+			}, 100);
+		});
+	} else {
+		window.moveTo(windowPersist.screenX, windowPersist.screenY);
+	}
+	window.addEventListener("unload", function() {
+		// save windowState if it's maximized or normal - otherwise use value
+		var ws = (window.windowState == 1 || window.windowState == 3) ? window.windowState : windowPersist.windowState;
+		// save the other stuff if it's normal state, otherwise use previous
+		if (ws == 3) {
+			// width and height get read from document but set from document.documentElement
+			// this can fail if this is in a tab. if so, don't save.
+			try {
+				windowPersist = {
+					width: window.outerWidth,
+					height: window.outerHeight,
+					screenX: window.screenX,
+					screenY: window.screenY
+				}
+			} catch (ex) {
+				return;
+			}
+		}
+		windowPersist.windowState = ws;
+
+		prefs.setCharPref("editorWindowPersist", JSON.stringify(windowPersist));
+	});
+}
+
 var sourceEditorType = null;
 var sourceEditor = null;
 function init() {
 	nameE = document.getElementById("name");
-	tagsE = document.getElementById("tags");
+	nameE.addEventListener("input", function() {enableSave(true);});
 	updateUrlE = document.getElementById("update-url")
+	updateUrlE.addEventListener("input", function() {enableSave(true);});
 	strings = document.getElementById("strings");
 	codeE = document.getElementById("internal-code");
 
+	initStyle();
+
 	if (prefs.getIntPref("editor") == 0) {
 		// sourceeditor, firefox 27+
 		let Editor = null;
 		if (require) {
-			try {
-				Editor = require("devtools/sourceeditor/editor");
+			try { // for Firefox 44 and later, bug 912121
+				Editor = require("devtools/client/sourceeditor/editor")
 			} catch (ex) {
-				//unavailable
+				try { // for Firefox 43 and earlier
+					Editor = require("devtools/sourceeditor/editor");
+				} catch (ex) {
+					//Components.utils.reportError(ex) //unavailable
+				}
 			}
 		}
 		if (Editor && ("modes" in Editor)) {
 			document.getElementById("itsalltext").style.visibility = "hidden";
+			// disable Firefox's built-in find
+			document.documentElement.setAttribute("disablefastfind", "true");
+			var extraKeys = {};
+			// This needs to be uppercase to work, some locales use lowercase to match a character in the label
+			extraKeys[Editor.accel(document.getElementById("save-button").getAttribute("accesskey").toUpperCase())] = save;
+			extraKeys["F3"] = "findNext";
+			extraKeys["Shift-F3"] = "findPrev";
 			sourceEditor = new Editor({
 				mode: Editor.modes.css,
 				lineNumbers: true,
-				contextMenu: "orion-context"
+				contextMenu: "sourceeditor-context",
+				value: style.code,
+				extraKeys: extraKeys,
+				autocomplete: true
 			});
 			var sourceEditorElement = document.getElementById("sourceeditor");
-			document.getElementById("editor").selectedIndex = 2;
+			document.getElementById("editor").selectedIndex = 1;
 			sourceEditorType = "sourceeditor";
 			sourceEditor.appendTo(sourceEditorElement).then(init2);
-			return;
-		}
-		
-		// orion, firefox 8-26
-		var obj = {};
-		try {
-			Components.utils.import("resource:///modules/source-editor.jsm", obj);
-		} catch (ex) {
-			try {
-				// (moved circa firefox 27)
-				Components.utils.import("resource:///modules/devtools/sourceeditor/source-editor.jsm", obj);
-			} catch (ex) {
-				// orion not available
-			}
-		}
-		// check orion's pref
-		if ("SourceEditor" in obj && Services.prefs.getCharPref(obj.SourceEditor.PREFS.COMPONENT) != "textarea") {
-			// use orion
-			sourceEditor = new obj.SourceEditor();
-			sourceEditorType = "orion";
-			initOrion();
+			sourceEditor.on("change", function(cm, changeObj) {
+				enableSave(true);
+				enablePreview(true);
+				enableCheckForErrors(true);
+			});
 			return;
 		}
 	}
 	// textbox
 	sourceEditorType = "textarea";
 	sourceEditor = codeE;
+	codeE.addEventListener("input", function() {
+		enableSave(true);
+		enablePreview(true);
+		enableCheckForErrors(true);
+	});
+
 	setTimeout(init2, 100);
 }
 
 function initStyle() {
 	var service = Components.classes["@userstyles.org/style;1"].getService(Components.interfaces.stylishStyle);
 
-	// See if the ID is in the URL
+	// See if the ID/code is in the URL
 	var id;
+	var code = null;
 	var urlParts = location.href.split("?");
 	if (urlParts.length > 1) {
+        let params;
 		params = urlParts[1].split("&");
 		params.forEach(function(param) {
 			var kv = param.split("=");
-			if (kv.length > 1 && kv[0] == "id") {
-				id = kv[1];
+			if (kv.length > 1) {
+				if (kv[0] == "id") {
+					id = decodeURIComponent(kv[1]);
+				} else if (kv[0] == "code") {
+					code = decodeURIComponent(kv[1]);
+				}
 			}
 		});
 	}
 	if (id) {
 		style = service.find(id, service.CALCULATE_META | service.REGISTER_STYLE_ON_CHANGE);
-
-	// See the arguments passed in
-	} else if (window.arguments) {
-		if ("id" in window.arguments[0]) {
-			style = service.find(window.arguments[0].id, service.CALCULATE_META | service.REGISTER_STYLE_ON_CHANGE);
-		} else if ("style" in window.arguments[0]) {
-			style = window.arguments[0].style;
-			style.mode = service.CALCULATE_META | service.REGISTER_STYLE_ON_CHANGE;
+		enableSave(false);
+		enablePreview(!style.enabled);
+		document.documentElement.setAttribute("windowtype", stylishCommon.getWindowName("stylishEdit", id));
+	} else {
+		if (code == null) {
+			code = "";
 		}
-		installPingURL = window.arguments[0].installPingURL;
-		installCallback = window.arguments[0].installCallback;
-		document.documentElement.setAttribute("windowtype", window.arguments[0].windowType);
+		style = Components.classes["@userstyles.org/style;1"].createInstance(Components.interfaces.stylishStyle);
+		style.mode = style.CALCULATE_META | style.REGISTER_STYLE_ON_CHANGE;
+		style.init(null, null, null, null, null, code, false, null, null, null);
+		enableSave(true);
+		enablePreview(true);
 	}
 
 	if (style) {
 		nameE.value = style.name;
-		tagsE.value = style.getMeta("tag", {}).join(" ");
 		updateUrlE.value = style.updateUrl;
-		codeElementWrapper.value = style.code;
-		// if the style already has an id, it's been previously saved, so this is an edit
-		// if the style has no id but has a url, it's an install
-		document.documentElement.getButton("extra1").hidden = style.id || !style.url;
-		if (style.id) {
-			document.title = strings.getFormattedString("editstyletitle", [style.name]);
-		} else {
-			document.title = strings.getString("newstyletitle");
-		}
+		updateTitle();
+	}
+}
+
+function updateTitle() {
+	// if the style already has an id, it's been previously saved, so this is an edit
+	// if the style has no id but has a url, it's an install
+	if (style.id) {
+		document.title = strings.getFormattedString("editstyletitle", [style.name]);
 	} else {
-		style = Components.classes["@userstyles.org/style;1"].createInstance(Components.interfaces.stylishStyle);
-		document.documentElement.getButton("extra1").hidden = true;
 		document.title = strings.getString("newstyletitle");
 	}
 }
 
-function initOrion() {
-		// orion and it's all text don't get along. it's all text will update display later, so let's use visibility
-		document.getElementById("itsalltext").style.visibility = "hidden";
-		
-		var orionElement = document.getElementById("orion");
-		sourceEditor.init(orionElement, {mode: sourceEditor.MODES.CSS, showLineNumbers: true}, init2);
-		document.getElementById("editor").selectedIndex = 1;
-		var appInfo = Components.classes["@mozilla.org/xre/app-info;1"].getService(Components.interfaces.nsIXULAppInfo);
-		var versionChecker = Components.classes["@mozilla.org/xpcom/version-comparator;1"].getService(Components.interfaces.nsIVersionComparator);
-		// before firefox 11, we need to set up our own undo key binding
-		if ((appInfo.ID == "{ec8030f7-c20a-464f-9b0e-13a3a9e97384}" || appInfo.ID == "{3550f703-e582-4d05-9a08-453d09bdfdc6}") && versionChecker.compare(appInfo.version, "11.0a1") < 0) {
-			orionElement.addEventListener("keypress", handleOrionUndo, false);
-		}
-		window.controllers.insertControllerAt(0, undoController);
-		// only use our custom undo
-		document.getElementById("menu_undo").style.display = "none";
-		document.getElementById("stylish_menu_undo").style.display = "";
-}
-
 function init2() {
 
-	if (sourceEditorType == "orion") {
-		sourceEditor.addEventListener("ContextMenu", handleOrionContext, false);
-	}
 	if (sourceEditorType == "textarea" || (sourceEditorType == "sourceeditor" && "setOption" in sourceEditor)) {
 		var wrapLines = prefs.getBoolPref("wrap_lines");
 		refreshWordWrap(wrapLines);
@@ -164,20 +190,11 @@ function init2() {
 		wrapLinesE.checked = wrapLines;
 		wrapLinesE.style.display = "";
 	}
-	var autocompleteEnabled = false;
-	try {
-		autocompleteEnabled = Services.prefs.getBoolPref("devtools.styleeditor.autocompletion-enabled");
-	} catch (ex) {}
-	if (sourceEditorType == "sourceeditor" && autocompleteEnabled) {
-		// Up to Firefox 28, sometimes "require" will return an object when something is not available instead of throwing.
-		// Rather than trying to detect if autocompleter is available, let's just try to use it.
-		try {
-			sourceEditor.extend(autocompleter);
-			sourceEditor.setupAutoCompletion(null);
-		} catch (ex) { }
-	}
 
-	initStyle();
+	// the initial value for sourceeditor is set in the Editor constructor, which has the benefit of not being undoable
+	if (sourceEditorType != "sourceeditor") {
+		codeElementWrapper.value = style.code;
+	}
 
 	setTimeout(function(){
 		// the code returned is different for some reason a little later...
@@ -187,20 +204,10 @@ function init2() {
 			codeElementWrapper.setSelectionRange(0, 0);
 		}
 	},100);
-}
 
-function handleOrionUndo(event) {
-	if (event.ctrlKey) {
-		if (event.which == 122 || event.which == 90) { // Z
-			if (event.shiftKey) {
-				sourceEditor.redo();
-			} else {
-				sourceEditor.undo()
-			}
-		} else if (event.which == 121 || event.which == 89) { // Y
-			sourceEditor.redo();
-		}
-	}
+	initFinder();
+
+	codeElementWrapper.focus();
 }
 
 var undoController = {
@@ -223,26 +230,6 @@ var undoController = {
 	onEvent: function() {}
 }
 
-function handleOrionContext(event) {
-	sourceEditor.focus();
-	goUpdateGlobalEditMenuItems();
-	goUpdateCommand("stylish_cmd_undo");
-	var menu = document.getElementById("orion-context");
-	if (menu.state == "closed") {
-		menu.openPopupAtScreen(event.screenX, event.screenY, true);
-	}
-}
-
-function switchToInstall() {
-	Services.prefs.setBoolPref("extensions.stylish.editOnInstall", false);
-	style.name = nameE.value;
-	if (codeElementWrapper.value != initialCode) {
-		style.code = codeElementWrapper.value;
-	}
-	stylishCommon.openInstall({style: style, installPingURL: installPingURL, installCallback: installCallback});
-	window.close();
-}
-
 function save() {
 	style.name = nameE.value;
 	if (!style.name) {
@@ -264,62 +251,49 @@ function save() {
 
 	if (code != initialCode) {
 		style.code = code;
+		initialCode = style.code
 	} else {
 		// we don't want to change the code, but we want to undo any preview
 		style.revert();
 	}
 
-	style.removeAllMeta("tag")
-	stylishCommon.cleanTags(tagsE.value).forEach(function(v) {
-		style.addMeta("tag", v);
-	});
 	style.updateUrl = updateUrlE.value;
+
+	var newStyle = !style.id;
+
 	style.save();
-	saved = true;
-	if (installPingURL) {
-		var req = new XMLHttpRequest();
-		req.open("GET", installPingURL, true);
-		stylishCommon.fixXHR(req);
-		req.send(null);
+
+	if (newStyle) {
+		location.href = location.href.split("?")[0] + "?id=" + style.id;
+		return true;
 	}
 
+	updateTitle();
+
+	enableSave(false);
+	enablePreview(!style.enabled);
+
 	return true;
 }
 
+function enableSave(enabled) {
+	document.getElementById("save-button").disabled = !enabled;
+}
+
+function enablePreview(enabled) {
+	document.getElementById("preview-button").disabled = !enabled;
+}
+
+function enableCheckForErrors(enabled) {
+	document.getElementById("check-for-errors-button").disabled = !enabled;
+}
+
 function preview() {
 	style.name = nameE.value;
 	style.code = codeElementWrapper.value;
-	checkForErrors();
 	// delay this so checkForErrors doesn't pick up on what happens
 	setTimeout(function() { style.setPreview(true);}, 50);
-}
-
-function cancelDialog() {
-	if (!saved && initialCode != codeElementWrapper.value) {
-		var ps = Components.interfaces.nsIPromptService
-		var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(ps);
-		switch (promptService.confirmEx(window, strings.getString("unsavedchangestitle"), strings.getString("unsavedchanges"), ps.BUTTON_POS_0 * ps.BUTTON_TITLE_SAVE + ps.BUTTON_POS_1 * ps.BUTTON_TITLE_DONT_SAVE + ps.BUTTON_POS_2 * ps.BUTTON_TITLE_CANCEL, "", "", "", null, {})) {
-			case 0:
-				return save();
-			case 1:
-				return true;
-			case 2:
-				return false;
-		}
-	}
-	return true;
-}
-
-function dialogClosing() {
-	//turn off preview!
-	style.setPreview(false);
-	if (!saved) {
-		style.revert();
-	}
-	
-	if (installCallback) {
-		installCallback(saved ? "success" : "cancelled");
-	}
+	enablePreview(false);
 }
 
 function checkForErrors() {
@@ -339,12 +313,12 @@ function checkForErrors() {
 				
 				// ignore other crap
 				if (error.category == "CSS Parser" && error.sourceName == "about:blank") {
-					var message = error.lineNumber + ":" + error.columnNumber + " " + error.errorMessage;
+					var newmessage = error.lineNumber + ":" + error.columnNumber + " " + error.errorMessage;
 					// don't duplicate
-					if (currentMessages.indexOf(message) == -1) {
-						currentMessages.push(message);
+					if (currentMessages.indexOf(newmessage) == -1) {
+						currentMessages.push(newmessage);
 						var label = document.createElementNS(stylishCommon.XULNS, "label");
-						label.appendChild(document.createTextNode(message));
+						label.appendChild(document.createTextNode(newmessage));
 						label.addEventListener("click", function() {goToLine(error.lineNumber, error.columnNumber) }, false);
 						errors.appendChild(label);
 					}
@@ -353,6 +327,7 @@ function checkForErrors() {
 		}
 	}
 	style.checkForErrors(codeElementWrapper.value, errorListener);
+	enableCheckForErrors(false);
 }
 
 function goToLine(line, col) {
@@ -436,7 +411,7 @@ function insertDataURI() {
 	var file = fp.file;
 	var contentType = cc["@mozilla.org/mime;1"].getService(ci.nsIMIMEService).getTypeFromFile(file);
 	var inputStream = cc["@mozilla.org/network/file-input-stream;1"].createInstance(ci.nsIFileInputStream);
-	inputStream.init(file, 0x01, 0600, 0);
+	inputStream.init(file, parseInt("01", 16), parseInt("0600", 8), 0);
 	var stream = cc["@mozilla.org/binaryinputstream;1"].createInstance(ci.nsIBinaryInputStream);
 	stream.setInputStream(inputStream);
 	var encoded = btoa(stream.readBytes(stream.available()));
@@ -578,14 +553,14 @@ if (finderJsmStyle) {
 
 var codeElementWrapper = {
 	get value() {
-		if (sourceEditorType == "orion" || sourceEditorType == "sourceeditor") {
+		if (sourceEditorType == "sourceeditor") {
 			return sourceEditor.getText();
 		}
 		return sourceEditor.value;
 	},
 
 	set value(v) {
-		if (sourceEditorType == "orion" || sourceEditorType == "sourceeditor") {
+		if (sourceEditorType == "sourceeditor") {
 			sourceEditor.setText(v);
 		} else {
 			sourceEditor.value = v;
@@ -593,9 +568,7 @@ var codeElementWrapper = {
 	},
 
 	setSelectionRange: function(start, end) {
-		if (sourceEditorType == "orion") {
-			sourceEditor.setSelection(start, end);
-		} else if (sourceEditorType == "sourceeditor") {
+		if (sourceEditorType == "sourceeditor") {
 			sourceEditor.setSelection(sourceEditor.getPosition(start), sourceEditor.getPosition(end));
 		} else {
 			sourceEditor.setSelectionRange(start, end);
@@ -607,9 +580,6 @@ var codeElementWrapper = {
 	},
 
 	get selectionStart() {
-		if (sourceEditorType == "orion") {
-			return sourceEditor.getSelection().start;
-		}
 		if (sourceEditorType == "sourceeditor") {
 			return sourceEditor.getOffset(sourceEditor.getCursor("start"));
 		}
@@ -617,9 +587,6 @@ var codeElementWrapper = {
 	},
 
 	get selectionEnd() {
-		if (sourceEditorType == "orion") {
-			return sourceEditor.getSelection().end;
-		}
 		if (sourceEditorType == "sourceeditor") {
 			return sourceEditor.getOffset(sourceEditor.getCursor("end"));
 		}
@@ -635,15 +602,12 @@ var codeElementWrapper = {
 	},
 
 	get scrollElement() {
-		if (sourceEditorType == "orion") {
-			return sourceEditor._view._viewDiv;
-		}
 		return sourceEditor.inputField;
 	}
 
 }
 
-window.addEventListener("load", function() {
+function initFinder() {
 	// sourceeditor has its own way of doing this
 	if (sourceEditorType != "sourceeditor") {
 		var findBar = document.getElementById("findbar");
@@ -667,7 +631,20 @@ window.addEventListener("load", function() {
 			event.preventDefault();
 		}
 	}, false);
-}, false);
+}
+
+// if the style we're editing has been changed, turn off preview
+var changeObserver = {
+	observe: function(subject, topic, data) {
+		if (subject.id == style.id) {
+			// unapply any preview and get our internal style object in sync with the changes
+			style.setPreview(false);
+			style.code = subject.code;
+			style.enabled = subject.enabled;
+		}
+	}
+};
+Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService).addObserver(changeObserver, "stylish-style-change", false);
 
 // if the style we're editing has been deleted, turn off preview and close the window
 var deleteObserver = {
@@ -675,10 +652,36 @@ var deleteObserver = {
 		if (subject.id == style.id) {
 			style.enabled = false;
 			style.setPreview(false);
-			// just so the user is not prompted to save
-			saved = true;
+			skipConfirmClosePrompt = true;
 			window.close();
 		}
 	}
 };
 Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService).addObserver(deleteObserver, "stylish-style-delete", false);
+
+// Prompt for unsaved changes.
+// This is firing twice in some cases like on Seamonkey, so don't do it if we've shown it in the past 5 seconds.
+var lastBeforeUnload = null;
+window.addEventListener("beforeunload", function(event) {
+	if (!skipConfirmClosePrompt && initialCode != codeElementWrapper.value && (lastBeforeUnload == null || Date.now() - lastBeforeUnload > 5000)) {
+		lastBeforeUnload = Date.now();
+		// Firefox will show its own stuff, so the text doesn't matter as long as it's text
+		event.returnValue = "You're going to lose your changes - close anyway?";
+	}
+});
+
+// Closing by closing the window (e.g. pressing the X when in windowed mode) doesn't fire beforeunload. Cancel the close event and close it ourselves so beforeunload runs.
+window.addEventListener("close", function(event) {
+	event.preventDefault();
+	window.close();
+})
+
+window.addEventListener("unload", function(event) {
+	//turn off preview!
+	style.setPreview(false);
+	if (initialCode != codeElementWrapper.value) {
+		style.revert();
+	}
+});
+
+window.addEventListener("load", init);
